#!/bin/bash
#
# A simle script to archive all recent Amanda level-0 backup files.
# Part of amanda-server-addons package.

AMANDAARCHIVE="/mnt/amanda"                     # Archive root dir
AMANDAROOT="/var/backups"                       # Amanda backups root dir
BASENAME="/usr/bin/basename"                    # basename command call
DD="/bin/dd"                                    # dd command call
DF="/bin/df"                                    # df command call
DU="/usr/bin/du"                                # du command call
EXIT_ERR="1"                                    # Exit code on error
FIND="/usr/bin/find"                            # find command call
GREP="/bin/grep"                                # grep command call
HEAD="/usr/bin/head"                            # head comand call
SED="/bin/sed"                                  # sed command call
SORT="/usr/bin/sort"                            # sort command call
TAIL="/usr/bin/tail"                            # tail command call

# Messages
MSG_AMANDA_LVL0_ARCHIVED="[OK: archived]"
MSG_AMANDA_LVL0_ARCHERR="[ERR: dd failed]"
MSG_AMANDA_LVL0_DELWARNINGS="Warnings during cleanup:"
MSG_AMANDA_LVL0_NOARCHIVE="Isn't an existing writable folder:"
MSG_AMANDA_LVL0_NOMOUNT="Isn't a mounted filesystem:"
MSG_AMANDA_LVL0_NOROOTDIR="Isn't a readable folder:"
MSG_AMANDA_LVL0_NOSPACE="[SKIP: low space!]"
MSG_AMANDA_LVL0_OPTION="Invalid option:"
MSG_AMANDA_LVL0_USAGE="Usage: $0 [-d | --delete] [-m --mountcheck] [archive_folder] [backups_folder]"

# Getting common variables.
HEADERS="/etc/default/amanda-server-addons"
if [ -f "$HEADERS" ]; then . $HEADERS; fi

# Checking options (beginning with - or --)
# -d | --delete: delete files in archive folder (if any)
# -m | --mountcheck: make sure the archive folder is a mount point.
delete=0; mountcheck=0
while [ "${1:0:1}" = "-" -o "${1:0:2}" = "--" ]
do
    case "$1" in
        "-d" | "--delete" )
            delete=1
        ;;
        "-m" | "--mountcheck" )
            mountcheck=1
        ;;
        * )
            echo -e "$MSG_AMANDA_LVL0_OPTION $1" >&2; exit $EXIT_ERR
        ;;
    esac
    shift
done

# Getting parameters or using defaults.
AMANDAARCHIVE=${1:-$AMANDAARCHIVE}; shift
AMANDAROOT=${1:-$AMANDAROOT}; shift

#
# Functions
#

# Selecting last saved level-0 archives from all Amanda backups.
function last_archives {

    local amandafile amandafiles amandalastfile amandapattern 
    # List of all available level-0 archives (by name convention).
    local lvl0_all=$($FIND -L "$AMANDAROOT" -name "?????.*.0")

    for amandafile in $lvl0_all
    do
        # Pattern below matches all archives containing the same
        # content saved at different time (days).
        amandapattern=$($BASENAME $amandafile | $SED "s/[0-9]\+\.//")
        amandafiles=$($FIND -L "$AMANDAROOT" -name "*.$amandapattern")
        # Last saved archive:
        amandalastfile=$(ls -1 --sort=time $amandafiles | $HEAD -n1)
        # Appends current amandafile to the output list when it is the last.
        if [ "$amandafile" = "$amandalastfile" ]; then echo "$amandafile"; fi
        # Otherwise skips them.
    done
};

#
# Main program
#

# Mountpoint check if necessary:
# AMANDAARCHIVE must be a mounted filesystem's mount point.
if [ $mountcheck -eq 1 ]; then
    if [ -z "$(cat /proc/mounts | $GREP " $AMANDAARCHIVE ")" ]; then
        echo "$MSG_AMANDA_LVL0_NOMOUNT $AMANDAARCHIVE" >&2; exit $EXIT_ERR
    fi
fi

# AMANDAROOT check at all:
# must be a readable folder.
if [ ! -d "$AMANDAROOT" ]; then
    echo "$MSG_AMANDA_LVL0_NOROOTDIR $AMANDAROOT" >&2; exit $EXIT_ERR
fi

# AMANDAARCHIVE check at all:
# must be a writable folder.
if [ ! -d "$AMANDAARCHIVE" ]; then
    echo "$MSG_AMANDA_LVL0_NOARCHIVE $AMANDAARCHIVE" >&2; exit $EXIT_ERR
fi
# Empty them if necessary (files only).
if [ $delete -eq 1 ]; then
    message=$(rm "$AMANDAARCHIVE"/* 2>&1); RESULT=$?
    if [ $RESULT -ne 0 ]; then
        echo -e "$MSG_AMANDA_LVL0_DELWARNINGS $AMANDAARCHIVE\n$message\n" >&2
    fi
fi

# Getting a list of all living level-0 Amanda backups (size and pathname);
# sorted by size, reverse order (biggest is first).
amandalist=$($DU --dereference-args --bytes $(last_archives) | $SORT -k1 -n -r)

# Processing list, line by line.
echo -e "$amandalist" | while read item
do
    # Parsing item, creating target filename.
    need_space=$(echo "$item" | $SED "s/\([0-9]\+\)\(\s\+\)\(.*\)/\1/")
    source_file=$(echo "$item" | $SED "s/\([0-9]\+\)\(\s\+\)\(.*\)/\3/")
    target_file="$AMANDAARCHIVE/$($BASENAME $source_file | $SED "s/[0-9]\+\.//").tgz"
    # Skipping . or .. pseudonames
    if [ "$source_file" = "." -o "$source_file" = ".." ]; then continue; fi
    # Recalculating free space on device containing AMANDAARCHIVE
    # with parsing standard 'df' function's output.
    free_space=$($DF --block-size=1 "$AMANDAARCHIVE" | $TAIL -n1 | \
                 $SED "s/\([^ ]\+\) \+\([0-9]\+\) \+\([0-9]\+\) \+\([0-9]\+\) \+.*/\4/")
    # Error message if isn't enough space.
    if [ $need_space -gt $free_space ]; then
        echo -e "$MSG_AMANDA_LVL0_NOSPACE\t$free_space\t$need_space\t$target_file" >&2
    else
        # Trying to copy the archive into regular tar.gz file 
        # skipping Amanda tape header.
        $DD if="$source_file" of="$target_file" bs=32k skip=1 status=noxfer >/dev/null 2>&1 ; RESULT=$?
        if [ $RESULT -eq 0 ]; then
            echo -e "$MSG_AMANDA_LVL0_ARCHIVED\t$free_space\t$need_space\t$target_file"
        else
            echo -e "$MSG_AMANDA_LVL0_ARCHERR\tERR:$RESULT\t$need_space\t$target_file" >&2
        fi
    fi
done

# That's all, folks ! :-)
