#!/bin/bash
#
# Simple script to initialize new Amanda task.
# Part of amanda-addons-server package

AMANDACONF=/etc/amanda          # Root directory for Amanda config files
AMANDAGROUP=disk                # Group of Amanda tapes
AMANDALIB=/var/lib/amanda       # Root directory for Amanda libraries
AMANDALOG=/var/log/amanda       # Root directory for Amanda log files
AMANDAROOT=/var/backups         # Root directory for Amanda backups
AMANDATAPES=14 			# Default number of tapes to use for a task
AMANDAUSER=backup               # User running Amanda
CRONDIR="/etc/cron.d"		# Root directory for cron jobs
CRONTEMPLATE="amanda-cron"	# Template file for cron jobs
DATE=`date '+%Y-%m-%d %H:%M'`   # Current date and time
ENVSUBST="/usr/bin/envsubst"    # envsubst command call
EXIT_ERR="1"			# Exit code on error
EXIT_SUCC="0"                   # Exit code on success
TAPELENGTH="20000"		# Default tape size (20GB)
TAPESET="01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21"
TEMPLATEDIR="/usr/local/share/amanda-server-addons/templates"

MSG_AMANDA_NEW_CREATED_CHG="Changer config file created:"
MSG_AMANDA_NEW_CREATED_CONF="Configuration file created:"
MSG_AMANDA_NEW_CREATED_CONFDIR="Configuration directory created:"
MSG_AMANDA_NEW_CREATED_CRON="Cron job created:"
MSG_AMANDA_NEW_CREATED_LIB="Libs root directory created:"
MSG_AMANDA_NEW_CREATED_LOG="Logs root directory created:"
MSG_AMANDA_NEW_CREATED_TAPE="Tape root directory created:"
MSG_AMANDA_NEW_CREATED_TAPELIST="Tapelist file created:"
MSG_AMANDA_NEW_NOCONFIG="Missing config file:"
MSG_AMANDA_NEW_ROOTNEED="Run as root, please."
MSG_AMANDA_NEW_USAGE="Usage: $0 Amanda_task_name Absolute_mount_point [No_of_tapes] (as root).\nMake sure you have a correct $AMANDACONF/Amanda_config_name/amanda.conf file."
MSG_AMANDA_NEW_WRONGTAPES="No. of tapes must be an integer between 1 and 21."

# Getting common variables.
HEADERS="/etc/default/amanda-server-addons"
if [ -f "$HEADERS" ]; then . $HEADERS; fi

# Getting and checking mandatory parameters.
CONFNAME=$1; shift
MOUNTPOINT=$1; shift
if [ -z "$CONFNAME" -o -z "$MOUNTPOINT" ] ; then
    echo -e "$MSG_AMANDA_NEW_USAGE" >&2
    exit $EXIT_ERR
fi

# Getting and checking optional parameters.
TAPES=${1:-$AMANDATAPES}; shift
TAPESET=${TAPESET:0:$((3*TAPES))}
if [ -z "$TAPESET" ]; then
    echo -e "$MSG_AMANDA_NEW_WRONGTAPES" >&2
    exit $EXIT_ERR
fi

# Checking user (root need).
if [ `whoami` != "root" ] ; then
    echo -e "$MSG_AMANDA_NEW_ROOTNEED" >&2
    exit $EXIT_ERR
fi

# Creating configuration directory, amanda.conf and an empty disklist.
if [ ! -d $AMANDACONF/$CONFNAME ]; then
    mkdir -m 770 $AMANDACONF/$CONFNAME
    chown $AMANDAUSER:$AMANDAUSER $AMANDACONF/$CONFNAME
    echo -e "$MSG_AMANDA_NEW_CREATED_CONFDIR $AMANDACONF/$CONFNAME"
fi
# amanda.conf by template.
if [ ! -f $AMANDACONF/$CONFNAME/amanda.conf ]; then
    export AMANDACONF AMANDALIB AMANDALOG AMANDAUSER CONFNAME DATE TAPELENGTH TAPES
    cat $TEMPLATEDIR/amanda.conf | $ENVSUBST > $AMANDACONF/$CONFNAME/amanda.conf
    chown $AMANDAUSER:$AMANDAUSER $AMANDACONF/$CONFNAME/amanda.conf
    chmod 640 $AMANDACONF/$CONFNAME/amanda.conf
    echo -e "$MSG_AMANDA_NEW_CREATED_CONF $AMANDACONF/$CONFNAME/amanda.conf"
fi
# empty disklist by template.
if [ ! -f $AMANDACONF/$CONFNAME/disklist ]; then
    export DATE
    cat $TEMPLATEDIR/disklist | $ENVSUBST > $AMANDACONF/$CONFNAME/disklist
    chown $AMANDAUSER:$AMANDAUSER $AMANDACONF/$CONFNAME/disklist
    chmod 640 $AMANDACONF/$CONFNAME/disklist
    echo -e "$MSG_AMANDA_NEW_CREATED_CONF $AMANDACONF/$CONFNAME/disklist"
fi

# Creating tape root directory for this config if absent.
if [ ! -d $MOUNTPOINT/$CONFNAME ] ; then
    mkdir $MOUNTPOINT/$CONFNAME
    chown $AMANDAUSER:$AMANDAGROUP $MOUNTPOINT/$CONFNAME
    chmod 770 $MOUNTPOINT/$CONFNAME
    echo "$MSG_AMANDA_NEW_CREATED_TAPE $MOUNTPOINT/$CONFNAME"
fi

# Creating lib root directory for this config if absent.
if [ ! -d $AMANDALIB/$CONFNAME ] ; then
    mkdir $AMANDALIB/$CONFNAME
    chown $AMANDAUSER:$AMANDAUSER $AMANDALIB/$CONFNAME
    chmod 770 $AMANDALIB/$CONFNAME
    echo "$MSG_AMANDA_NEW_CREATED_LIB $AMANDALIB/$CONFNAME"
fi

# Creating log root directory for this config if absent.
if [ ! -d $AMANDALOG/$CONFNAME ] ; then
    mkdir $AMANDALOG/$CONFNAME
    chown $AMANDAUSER:$AMANDAUSER $AMANDALOG/$CONFNAME
    chmod 770 $AMANDALOG/$CONFNAME
    echo "$MSG_AMANDA_NEW_CREATED_LOG $AMANDALOG/$CONFNAME"
fi

# Producing changer config file if absent.
if [ ! -f $AMANDACONF/$CONFNAME/changer.conf ] ; then
    touch $AMANDACONF/$CONFNAME/changer.conf
    chown $AMANDAUSER:$AMANDAGROUP $AMANDACONF/$CONFNAME/changer.conf
    chmod 600 $AMANDACONF/$CONFNAME/changer.conf
    cat > $AMANDACONF/$CONFNAME/changer.conf << "EOF"
# Amanda pseudotapes changer config file.
# Based on Amanda FAQ-O-Matic: How to configure for tapeless operation

multieject 0
gravity 0
needeject 0
ejectdelay 0

EOF
    echo "statefile $AMANDALIB/$CONFNAME/changer-status" >> $AMANDACONF/$CONFNAME/changer.conf
    echo "firstslot 01" >> $AMANDACONF/$CONFNAME/changer.conf
    for n in $TAPESET
    do
	echo ${n} > /dev/null
    done
    echo "lastslot ${n}" >> $AMANDACONF/$CONFNAME/changer.conf
    echo "$MSG_AMANDA_NEW_CREATED_CHG $AMANDACONF/$CONFNAME/changer.conf"
fi

# Producing tapelist if absent.
if [ ! -f $AMANDACONF/$CONFNAME/tapelist ] ; then
    touch $AMANDACONF/$CONFNAME/tapelist
    chown $AMANDAUSER:$AMANDAGROUP $AMANDACONF/$CONFNAME/tapelist
    chmod 600 $AMANDACONF/$CONFNAME/tapelist
    echo "$MSG_AMANDA_NEW_CREATED_TAPELIST $AMANDACONF/$CONFNAME/tapelist"
fi

# Creating and labeling virtual tapes.
for n in $TAPESET
do
    mkdir $MOUNTPOINT/$CONFNAME/tape${n}
    mkdir $MOUNTPOINT/$CONFNAME/tape${n}/data
    chown -R $AMANDAUSER:$AMANDAGROUP $MOUNTPOINT/$CONFNAME/tape${n}
    chmod -R 770 $MOUNTPOINT/$CONFNAME/tape${n}
    echo "slot ${n} file:$MOUNTPOINT/$CONFNAME/tape${n}" >> $AMANDACONF/$CONFNAME/changer.conf
    su $AMANDAUSER -c "/usr/sbin/amlabel $CONFNAME $CONFNAME${n} slot ${n}"
done

# Defining the cron job by template.
if [ ! -f $CRONDIR/$CRONTEMPLATE-$CONFNAME ]; then
    export CONFNAME DATE
    cat $TEMPLATEDIR/$CRONTEMPLATE | $ENVSUBST > $CRONDIR/$CRONTEMPLATE-$CONFNAME
    chown root:root $CRONDIR/$CRONTEMPLATE-$CONFNAME
    chmod 644 $CRONDIR/$CRONTEMPLATE-$CONFNAME
    echo -e "$MSG_AMANDA_NEW_CREATED_CRON $CRONDIR/$CRONTEMPLATE-$CONFNAME"
fi

# That's all, folks :-)
